// Here we add a constructor Tree::Tree to help initialize Trees properly. We
// also make the interface functions members of the struct in anticipation of
// making it a class. See bintree4.h for the full evolution into a class.
#pragma once

#include <memory>
#include <stdexcept>
#include <string>
#include <vector>

namespace bintree {

// A Tree now has a shared_ptr to a node and keeps track of its size.
struct Tree {
    struct Node;
    using link_t = std::shared_ptr<Node>;

    link_t root_;
    size_t size_;

    Tree();

    bool empty() const;
    size_t size() const;
    bool member(const std::string&) const;
    unsigned lookup(const std::string&) const;
    std::vector<std::string> get_keys() const;

    void insert(const std::string&, unsigned);
};

// An exception thrown by lookup(Tree, const std::string&) when the key isn't
// found.
struct Not_found : std::runtime_error
{
    Not_found(const std::string& key) :
            runtime_error("key not found: " + key) { }
};

}