#include "Rational.h"
#include <UnitTest++/UnitTest++.h>

using rational::Rational;

TEST(Rational_0)
{
    Rational r;
    CHECK_EQUAL(0, r.num);
    CHECK_EQUAL(1, r.den);
}

TEST(Rational_n)
{
    Rational r(5);
    CHECK_EQUAL(5, r.num);
    CHECK_EQUAL(1, r.den);
}

TEST(Rational_n_m)
{
    Rational r1(2, 3);
    CHECK_EQUAL(2, r1.num);
    CHECK_EQUAL(3, r1.den);

    Rational r2(4, 6);
    CHECK_EQUAL(2, r2.num);
    CHECK_EQUAL(3, r2.den);

    Rational r3(4, -6);
    CHECK_EQUAL(-2, r3.num);
    CHECK_EQUAL(3, r3.den);

    Rational r4(-4, -6);
    CHECK_EQUAL(2, r4.num);
    CHECK_EQUAL(3, r4.den);
}

TEST(Rational_den0_throws) {
    CHECK_THROW(Rational(5, 0), std::overflow_error);
}

TEST(Rational_equals)
{
    Rational r1(2, 3);
    Rational r2(4, 6);
    CHECK_EQUAL(r1, r2);
}

TEST(Rational_comparisons)
{
    Rational r1(3, 10);
    Rational r2(4, 10);

    CHECK(r1 < r2);
    CHECK(r1 <= r2);
    CHECK(r2 > r1);
    CHECK(r2 >= r1);
    CHECK(!(r1 > r2));
    CHECK(!(r1 >= r2));
    CHECK(!(r2 < r1));
    CHECK(!(r2 <= r1));
    CHECK(r1 <= r1);
    CHECK(r2 <= r2);
    CHECK(r1 >= r1);
    CHECK(r2 >= r2);
}

TEST(Format)
{
    Rational r1(5);
    Rational r2(5, 6);
    std::ostringstream fmt;
    fmt << r1 << ' ' << r2;
    CHECK_EQUAL("5 5/6", fmt.str());
}

TEST(To_double)
{
    Rational r1(5);
    Rational r2(1, 3);

    CHECK_EQUAL(5, to_double(r1));
    CHECK_CLOSE(0.3333, to_double(r2), 0.0001);
}

TEST(Negate)
{
    Rational r1(5);
    Rational r2(1, 3);

    CHECK_EQUAL(Rational(-5), -r1);
    CHECK_EQUAL(Rational(-1, 3), -r2);
}

TEST(Reciprocal)
{
    Rational r1(5);
    Rational r2(1, 3);

    CHECK_EQUAL(Rational(1, 5), reciprocal(r1));
    CHECK_EQUAL(Rational(3), reciprocal(r2));
}

TEST(Reciprocal_0_throws)
{
    Rational r(0);
    CHECK_THROW(reciprocal(r), std::overflow_error);
}

TEST(Rational_addition)
{
    using R = Rational;

    CHECK_EQUAL(R(11), R(5) + R(6));
    CHECK_EQUAL(R(2,3), R(1,3) + R(1,3));
    CHECK_EQUAL(R(7,12), R(1,3) + R(1,4));
    CHECK_EQUAL(R(1), R(5,12) + R(7,12));
    CHECK_EQUAL(R(35, 12), R(2, 3) + R(9, 4));
    CHECK_EQUAL(R(3, 8), R(1, 8) + R(1, 4));
}

TEST(Rational_multiplication)
{
    using R = Rational;

    CHECK_EQUAL(R(12), R(3) * R(4));
    CHECK_EQUAL(R(1), R(1,3) * R(3));
    CHECK_EQUAL(R(1), R(2,3) * R(3,2));
}

